from dbt.tests.adapter.empty.test_empty import BaseTestEmpty, BaseTestEmptyInlineSourceRef


class TestDuckDBEmpty(BaseTestEmpty):
    pass


class TestDuckDBEmptyInlineSourceRef(BaseTestEmptyInlineSourceRef):
    pass
